@@ -211,7 +211,7 @@ GSList *
 startup_list_read (gchar *name)
 {
   GHashTable *clients;
-  const char * const * system_config_dirs;
+  const char * const * system_dirs;
   char *path;
   int i;
   GSList *result = NULL;
@@ -220,10 +220,19 @@ startup_list_read (gchar *name)
   clients = g_hash_table_new (g_str_hash, g_str_equal);
 
   /* read directories */
-  system_config_dirs = g_get_system_config_dirs ();
-  for (i = 0; system_config_dirs[i] != NULL; i++)
+  system_dirs = g_get_system_data_dirs ();
+  for (i = 0; system_dirs[i] != NULL; i++)
     {
-      path = g_build_filename (system_config_dirs[i], "autostart", NULL);
+      path = g_build_filename (system_dirs[i], "gnome", "autostart", NULL);
+      search_desktop_entries_in_dir (clients, path);
+      g_free (path);
+    }
+
+  /* support old place (/etc/xdg/autostart) */
+  system_dirs = g_get_system_config_dirs ();
+  for (i = 0; system_dirs[i] != NULL; i++)
+    {
+      path = g_build_filename (system_dirs[i], "autostart", NULL);
       search_desktop_entries_in_dir (clients, path);
       g_free (path);
     }
@@ -262,7 +271,8 @@ startup_list_write (GSList *sl, const gchar *name)
 
       if (client->to_remove)
         {
-	  g_remove (client->desktop_file);
+	  if (g_file_test (client->desktop_file, G_FILE_TEST_EXISTS))
+	    g_remove (client->desktop_file);
 	}
       else
         {
@@ -281,9 +291,9 @@ startup_list_write (GSList *sl, const gchar *name)
 	    g_warning ("Could not save %s file\n", client->desktop_file);
 
 	  gnome_desktop_item_unref (ditem);
-
-	  tmp_list = tmp_list->next;
 	}
+
+      tmp_list = tmp_list->next;
     }
   g_free (dir);
 }
@@ -490,11 +500,11 @@ startup_list_add_dialog (GSList **sl, GtkWidget **dialog, GtkWidget *parent_dlg)
       int i = 2;
 
       basename = g_path_get_basename (client->argv[0]);
-      orig_filename = g_strdup_printf ("%s/.config/autostart/%s.desktop", g_get_home_dir (), basename);
-      filename = g_strdup (orig_filename);
+      orig_filename = g_build_filename (g_get_user_config_dir (), "autostart", basename, NULL);
+      filename = g_strdup_printf ("%s.desktop", orig_filename);
       while (g_file_exists (filename))
         {
-	  char *tmp = g_strdup_printf ("%s-%d", orig_filename, i);
+	  char *tmp = g_strdup_printf ("%s-%d.desktop", orig_filename, i);
 
 	  g_free (filename);
 	  filename = tmp;
@@ -567,22 +577,49 @@ startup_list_enable (GSList **sl, GtkTreeModel *model, GtkTreeSelection *sel)
   if (!client->enabled)
     {
      char *path, *basename;
+     const char * const * system_dirs;
+      int i;
 
      basename = g_path_get_basename (client->desktop_file);
 
       /* if the desktop file is in the user's home and there is one file with the same
 	 name in the system-wide dir, just remove it */
-     path = g_build_filename (PREFIX, "share", "autostart", basename, NULL);
-     if (g_str_has_prefix (client->desktop_file, g_get_home_dir ())
-	 && g_file_test (path, G_FILE_TEST_EXISTS))
-       client->to_remove = TRUE;
+     if (g_str_has_prefix (client->desktop_file, g_get_user_config_dir ())) {
+       system_dirs = g_get_system_data_dirs ();
+       for (i = 0; system_dirs[i] != NULL && !client->to_remove; i++)
+         {
+           path = g_build_filename (system_dirs[i], "gnome", "autostart", basename, NULL);
+           if (g_file_test (path, G_FILE_TEST_EXISTS))
+	     {
+               client->to_remove = TRUE;
+	       g_free (path);
+	       break;
+	     }
+
+           g_free (path);
+         }
+
+       /* support old place (/etc/xdg/autostart) */
+       system_dirs = g_get_system_config_dirs ();
+       for (i = 0; system_dirs[i] != NULL && !client->to_remove; i++)
+         {
+           path = g_build_filename (system_dirs[i], "autostart", basename, NULL);
+           if (g_file_test (path, G_FILE_TEST_EXISTS))
+             {
+               client->to_remove = TRUE;
+	       g_free (path);
+	       break;
+	     }
+
+           g_free (path);
+         }
+     }
 
      client->enabled = TRUE;
 
      spc_write_state ();
 
      g_free (basename);
-     g_free (path);
     }
 }
 
@@ -598,17 +635,39 @@ startup_list_disable (GSList **sl, GtkTreeModel *model, GtkTreeSelection *sel)
   if (client->enabled)
     {
       char *path, *basename;
+      const char * const * system_dirs;
+      int i;
 
       basename = g_path_get_basename (client->desktop_file);
 
       /* if the desktop file is in the system-wide dir, create another one on
        the user's home */
-      if (g_str_has_prefix (client->desktop_file, PREFIX))
+      system_dirs = g_get_system_data_dirs ();
+      for (i = 0; system_dirs[i] != NULL && !client->to_remove; i++)
         {
-          path = g_build_filename (g_get_home_dir (), ".config", "autostart", basename, NULL);
-	  g_free (client->desktop_file);
-	  client->desktop_file = path;
-	}
+          if (g_str_has_prefix (client->desktop_file, system_dirs[i]))
+            {
+              path = g_build_filename (g_get_user_config_dir (), "autostart", basename, NULL);
+              g_free (client->desktop_file);
+              client->desktop_file = path;
+
+	      break;
+            }
+        }
+
+      /* support old place (/etc/xdg/autostart) */
+      system_dirs = g_get_system_config_dirs ();
+      for (i = 0; system_dirs[i] != NULL && !client->to_remove; i++)
+        {
+          if (g_str_has_prefix (client->desktop_file, system_dirs[i]))
+            {
+              path = g_build_filename (g_get_user_config_dir (), "autostart", basename, NULL);
+              g_free (client->desktop_file);
+              client->desktop_file = path;
+
+	      break;
+            }
+        }
 
       client->enabled = FALSE;
       spc_write_state ();
